@@ -5,7 +5,7 @@
  * Copyright (C) 2015 Pavol Bauer
  * Copyright (C) 2017 -- 2019 Robin Eriksson
  * Copyright (C) 2015 -- 2019 Stefan Engblom
- * Copyright (C) 2015 -- 2024 Stefan Widgren
+ * Copyright (C) 2015 -- 2025 Stefan Widgren
  *
  * SimInf is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,34 +21,110 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#include <Rinternals.h>
+#include "SimInf.h"
 #include <R_ext/Rdynload.h>
 #include <R_ext/Visibility.h>
-#include "SimInf.h"
+#include <Rinternals.h>
 
 /* Declare functions to register */
-SEXP SEIR_run(SEXP, SEXP);
-SEXP SIR_run(SEXP, SEXP);
-SEXP SIS_run(SEXP, SEXP);
-SEXP SISe_run(SEXP, SEXP);
-SEXP SISe3_run(SEXP, SEXP);
-SEXP SISe3_sp_run(SEXP, SEXP);
-SEXP SISe_sp_run(SEXP, SEXP);
-SEXP SimInf_abc_proposals(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-SEXP SimInf_abc_weights(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-SEXP SimInf_clean_indiv_events(SEXP, SEXP, SEXP, SEXP, SEXP);
-SEXP SimInf_distance_matrix(SEXP, SEXP, SEXP, SEXP, SEXP);
-SEXP SimInf_have_openmp(void);
-SEXP SimInf_init_threads(SEXP);
-SEXP SimInf_ldata_sp(SEXP, SEXP, SEXP);
-SEXP SimInf_split_events(SEXP, SEXP);
-SEXP SimInf_systematic_resampling(SEXP);
-SEXP SimInf_trajectory(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP SEIR_run(
+    SEXP,
+    SEXP);
 
-#define CALLDEF(name, n) {#name, (DL_FUNC) &name, n}
+SEXP SIR_run(
+    SEXP,
+    SEXP);
 
-static const R_CallMethodDef callMethods[] =
-{
+SEXP SIS_run(
+    SEXP,
+    SEXP);
+
+SEXP SISe_run(
+    SEXP,
+    SEXP);
+
+SEXP SISe3_run(
+    SEXP,
+    SEXP);
+
+SEXP SISe3_sp_run(
+    SEXP,
+    SEXP);
+
+SEXP SISe_sp_run(
+    SEXP,
+    SEXP);
+
+SEXP SimInf_abc_proposals(
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP);
+
+SEXP SimInf_abc_weights(
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP);
+
+SEXP SimInf_distance_matrix(
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP);
+
+SEXP SimInf_have_openmp(
+    void);
+
+SEXP SimInf_individual_events(
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP);
+
+SEXP SimInf_init_threads(
+    SEXP);
+
+SEXP SimInf_lambertW0(
+    SEXP);
+
+SEXP SimInf_ldata_sp(
+    SEXP,
+    SEXP,
+    SEXP);
+
+SEXP SimInf_split_events(
+    SEXP,
+    SEXP);
+
+SEXP SimInf_systematic_resampling(
+    SEXP);
+
+SEXP SimInf_trajectory(
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP,
+    SEXP);
+
+#define CALLDEF(name, n) {#name, (DL_FUNC) &(name), (n)}
+
+static const R_CallMethodDef callMethods[] = {
     CALLDEF(SEIR_run, 2),
     CALLDEF(SIR_run, 2),
     CALLDEF(SISe_run, 2),
@@ -58,15 +134,16 @@ static const R_CallMethodDef callMethods[] =
     CALLDEF(SISe_sp_run, 2),
     CALLDEF(SimInf_abc_proposals, 8),
     CALLDEF(SimInf_abc_weights, 7),
-    CALLDEF(SimInf_clean_indiv_events, 5),
+    CALLDEF(SimInf_individual_events, 5),
     CALLDEF(SimInf_distance_matrix, 5),
     CALLDEF(SimInf_have_openmp, 0),
     CALLDEF(SimInf_init_threads, 1),
+    CALLDEF(SimInf_lambertW0, 1),
     CALLDEF(SimInf_ldata_sp, 3),
     CALLDEF(SimInf_split_events, 2),
     CALLDEF(SimInf_systematic_resampling, 1),
-    CALLDEF(SimInf_trajectory, 10),
-    {NULL, NULL, 0}
+    CALLDEF(SimInf_trajectory, 11),
+    { NULL, NULL, 0 }
 };
 
 /**
@@ -74,7 +151,7 @@ static const R_CallMethodDef callMethods[] =
  *
  * @param info Information about the DLL being loaded
  */
-void attribute_visible
+attribute_visible void
 R_init_SimInf(
     DllInfo *info)
 {
@@ -82,10 +159,9 @@ R_init_SimInf(
     R_useDynamicSymbols(info, FALSE);
     R_forceSymbols(info, TRUE);
     R_RegisterCCallable("SimInf", "SimInf_local_spread",
-                        (DL_FUNC) &SimInf_local_spread);
+                        (DL_FUNC) & SimInf_local_spread);
     R_RegisterCCallable("SimInf", "SimInf_forward_euler_linear_decay",
-                        (DL_FUNC) &SimInf_forward_euler_linear_decay);
-    R_RegisterCCallable("SimInf", "SimInf_run",
-                        (DL_FUNC) &SimInf_run);
+                        (DL_FUNC) & SimInf_forward_euler_linear_decay);
+    R_RegisterCCallable("SimInf", "SimInf_run", (DL_FUNC) & SimInf_run);
     SimInf_init_threads(R_NilValue);
 }
