@@ -5,7 +5,7 @@
  * Copyright (C) 2015 Pavol Bauer
  * Copyright (C) 2017 -- 2019 Robin Eriksson
  * Copyright (C) 2015 -- 2019 Stefan Engblom
- * Copyright (C) 2015 -- 2022 Stefan Widgren
+ * Copyright (C) 2015 -- 2025 Stefan Widgren
  *
  * SimInf is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,21 +21,22 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#include <R_ext/Visibility.h>
 #include "SimInf.h"
+#include <R_ext/Visibility.h>
 
 /* Offset in integer compartment state vector */
-enum {S_1, I_1, S_2, I_2, S_3, I_3};
+enum { S_1, I_1, S_2, I_2, S_3, I_3 };
 
 /* Offset in real-valued continuous state vector */
-enum {PHI};
+enum { PHI };
 
 /* Offsets in node local data (ldata) to parameters in the model */
-enum {END_T1, END_T2, END_T3, END_T4};
+enum { END_T1, END_T2, END_T3, END_T4 };
 
 /* Offsets in global data (gdata) to parameters in the model */
-enum {UPSILON_1, UPSILON_2, UPSILON_3, GAMMA_1, GAMMA_2, GAMMA_3,
-      ALPHA, BETA_T1, BETA_T2, BETA_T3, BETA_T4, EPSILON};
+enum { UPSILON_1, UPSILON_2, UPSILON_3, GAMMA_1, GAMMA_2, GAMMA_3,
+    ALPHA, BETA_T1, BETA_T2, BETA_T3, BETA_T4, EPSILON
+};
 
 /**
  * In age category 1; susceptible to infected: S -> I
@@ -209,7 +210,7 @@ SISe3_post_time_step(
     int node,
     double t)
 {
-    const int day = (int)t % 365;
+    const int day = (int) t % 365;
     const double I_n = u[I_1] + u[I_2] + u[I_3];
     const double n = I_n + u[S_1] + u[S_2] + u[S_3];
     const double phi = v[PHI];
@@ -218,10 +219,15 @@ SISe3_post_time_step(
 
     /* Time dependent beta in each of the four intervals of the
      * year. Forward Euler step. */
-    v_new[PHI] = SimInf_forward_euler_linear_decay(
-        phi, day,
-        ldata[END_T1], ldata[END_T2], ldata[END_T3], ldata[END_T4],
-        gdata[BETA_T1], gdata[BETA_T2], gdata[BETA_T3], gdata[BETA_T4]);
+    v_new[PHI] = SimInf_forward_euler_linear_decay(phi, day,
+                                                   ldata[END_T1],
+                                                   ldata[END_T2],
+                                                   ldata[END_T3],
+                                                   ldata[END_T4],
+                                                   gdata[BETA_T1],
+                                                   gdata[BETA_T2],
+                                                   gdata[BETA_T3],
+                                                   gdata[BETA_T4]);
 
     if (n > 0.0)
         v_new[PHI] += gdata[ALPHA] * I_n / n + gdata[EPSILON];
@@ -232,7 +238,7 @@ SISe3_post_time_step(
         return SIMINF_ERR_V_IS_NOT_FINITE;
     if (v_new[PHI] < 0.0)
         return SIMINF_ERR_V_IS_NEGATIVE;
-    return phi != v_new[PHI]; /* 1 if needs update */
+    return phi != v_new[PHI];   /* 1 if needs update */
 }
 
 /**
@@ -242,14 +248,15 @@ SISe3_post_time_step(
  * @param solver The numerical solver.
  * @return The simulated trajectory.
  */
-SEXP attribute_hidden
+attribute_hidden SEXP
 SISe3_run(
     SEXP model,
     SEXP solver)
 {
-    TRFun tr_fun[] = {&SISe3_S_1_to_I_1, &SISe3_I_1_to_S_1,
-                      &SISe3_S_2_to_I_2, &SISe3_I_2_to_S_2,
-                      &SISe3_S_3_to_I_3, &SISe3_I_3_to_S_3};
+    TRFun tr_fun[] = { &SISe3_S_1_to_I_1, &SISe3_I_1_to_S_1,
+        &SISe3_S_2_to_I_2, &SISe3_I_2_to_S_2,
+        &SISe3_S_3_to_I_3, &SISe3_I_3_to_S_3
+    };
 
     return SimInf_run(model, solver, tr_fun, &SISe3_post_time_step);
 }
